@@ -5,14 +5,13 @@
 
 """Parser for PDB files."""
 
-# For using with statement in Python 2.5 or Jython
-from __future__ import with_statement
+from __future__ import print_function
 
 import warnings
 
 try:
     import numpy
-except:
+except ImportError:
     from Bio import MissingPythonDependencyError
     raise MissingPythonDependencyError(
         "Install NumPy if you want to use the PDB parser.")
@@ -30,31 +29,30 @@ from Bio.PDB.parse_pdb_header import _parse_pdb_header_list
 
 
 class PDBParser(object):
-    """
-    Parse a PDB file and return a Structure object.
-    """
+    """Parse a PDB file and return a Structure object."""
 
     def __init__(self, PERMISSIVE=True, get_header=False,
                  structure_builder=None, QUIET=False):
-        """
+        """Create a PDBParser object.
+
         The PDB parser call a number of standard methods in an aggregated
         StructureBuilder object. Normally this object is instanciated by the
         PDBParser object itself, but if the user provides his/her own
         StructureBuilder object, the latter is used instead.
 
         Arguments:
+         - PERMISSIVE - Evaluated as a Boolean. If false, exceptions in
+           constructing the SMCRA data structure are fatal. If true (DEFAULT),
+           the exceptions are caught, but some residues or atoms will be missing.
+           THESE EXCEPTIONS ARE DUE TO PROBLEMS IN THE PDB FILE!.
+         - get_header - unused argument kept for historical compatibilty.
+         - structure_builder - an optional user implemented StructureBuilder class.
+         - QUIET - Evaluated as a Boolean. If true, warnings issued in constructing
+           the SMCRA data will be suppressed. If false (DEFAULT), they will be shown.
+           These warnings might be indicative of problems in the PDB file!
 
-        o PERMISSIVE - Evaluated as a Boolean. If false, exceptions in
-        constructing the SMCRA data structure are fatal. If true (DEFAULT),
-        the exceptions are caught, but some residues or atoms will be missing.
-        THESE EXCEPTIONS ARE DUE TO PROBLEMS IN THE PDB FILE!.
-
-        o structure_builder - an optional user implemented StructureBuilder class.
-
-        o QUIET - Evaluated as a Boolean. If true, warnings issued in constructing
-        the SMCRA data will be suppressed. If false (DEFAULT), they will be shown.
-        These warnings might be indicative of problems in the PDB file!
         """
+        # get_header is not used but is left in for API compatibility
         if structure_builder is not None:
             self.structure_builder = structure_builder
         else:
@@ -71,50 +69,47 @@ class PDBParser(object):
         """Return the structure.
 
         Arguments:
-        o id - string, the id that will be used for the structure
-        o file - name of the PDB file OR an open filehandle
+         - id - string, the id that will be used for the structure
+         - file - name of the PDB file OR an open filehandle
+
         """
+        with warnings.catch_warnings():
+            if self.QUIET:
+                warnings.filterwarnings("ignore", category=PDBConstructionWarning)
 
-        if self.QUIET:
-            warning_list = warnings.filters[:]
-            warnings.filterwarnings("ignore", category=PDBConstructionWarning)
+            self.header = None
+            self.trailer = None
+            # Make a StructureBuilder instance (pass id of structure as parameter)
+            self.structure_builder.init_structure(id)
 
-        self.header = None
-        self.trailer = None
-        # Make a StructureBuilder instance (pass id of structure as parameter)
-        self.structure_builder.init_structure(id)
+            with as_handle(file, mode='rU') as handle:
+                self._parse(handle.readlines())
 
-        with as_handle(file) as handle:
-            self._parse(handle.readlines())
-
-        self.structure_builder.set_header(self.header)
-        # Return the Structure instance
-        structure = self.structure_builder.get_structure()
-
-        if self.QUIET:
-            warnings.filters = warning_list
+            self.structure_builder.set_header(self.header)
+            # Return the Structure instance
+            structure = self.structure_builder.get_structure()
 
         return structure
 
     def get_header(self):
-        "Return the header."
+        """Return the header."""
         return self.header
 
     def get_trailer(self):
-        "Return the trailer."
+        """Return the trailer."""
         return self.trailer
 
     # Private methods
 
     def _parse(self, header_coords_trailer):
-        "Parse the PDB file."
+        """Parse the PDB file (PRIVATE)."""
         # Extract the header; return the rest of the file
         self.header, coords_trailer = self._get_header(header_coords_trailer)
         # Parse the atomic data; return the PDB file trailer
         self.trailer = self._parse_coordinates(coords_trailer)
 
     def _get_header(self, header_coords_trailer):
-        "Get the header of the PDB file, return the rest."
+        """Get the header of the PDB file, return the rest (PRIVATE)."""
         structure_builder = self.structure_builder
         i = 0
         for i in range(0, len(header_coords_trailer)):
@@ -131,7 +126,7 @@ class PDBParser(object):
         return header_dict, coords_trailer
 
     def _parse_coordinates(self, coords_trailer):
-        "Parse the atomic data in the PDB file."
+        """Parse the atomic data in the PDB file (PRIVATE)."""
         local_line_counter = 0
         structure_builder = self.structure_builder
         current_model_id = 0
@@ -142,7 +137,7 @@ class PDBParser(object):
         current_residue_id = None
         current_resname = None
         for i in range(0, len(coords_trailer)):
-            line = coords_trailer[i]
+            line = coords_trailer[i].rstrip('\n')
             record_type = line[0:6]
             global_line_counter = self.line_counter + local_line_counter + 1
             structure_builder.set_line_counter(global_line_counter)
@@ -167,7 +162,7 @@ class PDBParser(object):
                 chainid = line[21]
                 try:
                     serial_number = int(line[6:11])
-                except:
+                except Exception:
                     serial_number = 0
                 resseq = int(line[22:26].split()[0])  # sequence identifier
                 icode = line[26]  # insertion code
@@ -184,7 +179,7 @@ class PDBParser(object):
                     x = float(line[30:38])
                     y = float(line[38:46])
                     z = float(line[46:54])
-                except:
+                except Exception:
                     # Should we allow parsing to continue in permissive mode?
                     # If so, what coordinates should we default to?  Easier to abort!
                     raise PDBConstructionException("Invalid or missing coordinate(s) at line %i."
@@ -193,18 +188,24 @@ class PDBParser(object):
                 # occupancy & B factor
                 try:
                     occupancy = float(line[54:60])
-                except:
+                except Exception:
                     self._handle_PDB_exception("Invalid or missing occupancy",
                                                global_line_counter)
-                    occupancy = None # Rather than arbitrary zero or one
+                    occupancy = None  # Rather than arbitrary zero or one
+                if occupancy is not None and occupancy < 0:
+                    # TODO - Should this be an error in strict mode?
+                    # self._handle_PDB_exception("Negative occupancy",
+                    #                            global_line_counter)
+                    # This uses fixed text so the warning occurs once only:
+                    warnings.warn("Negative occupancy in one or more atoms", PDBConstructionWarning)
                 try:
                     bfactor = float(line[60:66])
-                except:
+                except Exception:
                     self._handle_PDB_exception("Invalid or missing B factor",
                                                global_line_counter)
                     bfactor = 0.0  # The PDB use a default of zero if the data is missing
                 segid = line[72:76]
-                element = line[76:78].strip()
+                element = line[76:78].strip().upper()
                 if current_segid != segid:
                     current_segid = segid
                     structure_builder.init_seg(current_segid)
@@ -215,31 +216,31 @@ class PDBParser(object):
                     current_resname = resname
                     try:
                         structure_builder.init_residue(resname, hetero_flag, resseq, icode)
-                    except PDBConstructionException, message:
+                    except PDBConstructionException as message:
                         self._handle_PDB_exception(message, global_line_counter)
                 elif current_residue_id != residue_id or current_resname != resname:
                     current_residue_id = residue_id
                     current_resname = resname
                     try:
                         structure_builder.init_residue(resname, hetero_flag, resseq, icode)
-                    except PDBConstructionException, message:
+                    except PDBConstructionException as message:
                         self._handle_PDB_exception(message, global_line_counter)
                 # init atom
                 try:
                     structure_builder.init_atom(name, coord, bfactor, occupancy, altloc,
                                                 fullname, serial_number, element)
-                except PDBConstructionException, message:
+                except PDBConstructionException as message:
                     self._handle_PDB_exception(message, global_line_counter)
             elif record_type == "ANISOU":
-                anisou = map(float, (line[28:35], line[35:42], line[43:49],
-                                     line[49:56], line[56:63], line[63:70]))
+                anisou = [float(x) for x in (line[28:35], line[35:42], line[43:49],
+                                             line[49:56], line[56:63], line[63:70])]
                 # U's are scaled by 10^4
                 anisou_array = (numpy.array(anisou, "f") / 10000.0).astype("f")
                 structure_builder.set_anisou(anisou_array)
             elif record_type == "MODEL ":
                 try:
                     serial_num = int(line[10:14])
-                except:
+                except Exception:
                     self._handle_PDB_exception("Invalid or missing model serial number",
                                                global_line_counter)
                     serial_num = 0
@@ -258,15 +259,15 @@ class PDBParser(object):
                 current_residue_id = None
             elif record_type == "SIGUIJ":
                 # standard deviation of anisotropic B factor
-                siguij = map(float, (line[28:35], line[35:42], line[42:49],
-                                     line[49:56], line[56:63], line[63:70]))
+                siguij = [float(x) for x in (line[28:35], line[35:42], line[42:49],
+                                             line[49:56], line[56:63], line[63:70])]
                 # U sigma's are scaled by 10^4
                 siguij_array = (numpy.array(siguij, "f") / 10000.0).astype("f")
                 structure_builder.set_siguij(siguij_array)
             elif record_type == "SIGATM":
                 # standard deviation of atomic positions
-                sigatm = map(float, (line[30:38], line[38:45], line[46:54],
-                                     line[54:60], line[60:66]))
+                sigatm = [float(x) for x in (line[30:38], line[38:45], line[46:54],
+                                             line[54:60], line[60:66])]
                 sigatm_array = numpy.array(sigatm, "f")
                 structure_builder.set_sigatm(sigatm_array)
             local_line_counter += 1
@@ -275,7 +276,8 @@ class PDBParser(object):
         return []
 
     def _handle_PDB_exception(self, message, line_counter):
-        """
+        """Handle exception (PRIVATE).
+
         This method catches an exception that occurs in the StructureBuilder
         object (if PERMISSIVE), or raises it again, this time adding the
         PDB line number to the error message.
@@ -290,28 +292,3 @@ class PDBParser(object):
         else:
             # exceptions are fatal - raise again with new message (including line nr)
             raise PDBConstructionException(message)
-
-
-if __name__ == "__main__":
-
-    import sys
-
-    p = PDBParser(PERMISSIVE=True)
-
-    filename = sys.argv[1]
-    s = p.get_structure("scr", filename)
-
-    for m in s:
-        p = m.get_parent()
-        assert(p is s)
-        for c in m:
-            p = c.get_parent()
-            assert(p is m)
-            for r in c:
-                print r
-                p = r.get_parent()
-                assert(p is c)
-                for a in r:
-                    p = a.get_parent()
-                    if not p is r:
-                        print p, r
