@@ -24,12 +24,11 @@
 """Convert a series of Rebase files into a Restriction_Dictionary.py module.
 
 The Rebase files are in the emboss format:
+ - `emboss_e.###` - contains information about the restriction sites.
+ - `emboss_r.###` - contains general information about the enzymes.
+ - `emboss_s.###` - contains information about the suppliers.
 
-    emboss_e.###    -> contains information about the restriction sites.
-    emboss_r.###    -> contains general information about the enzymes.
-    emboss_s.###    -> contains information about the suppliers.
-
-Here ### is the 3 digit number REBASE release number (e.g. 312). The first
+Here `###` is the 3 digit number REBASE release number (e.g. 312). The first
 digit is the last digit of the year (e.g. 3 for 2013) and the two last the
 month (e.g. 12 for December).
 
@@ -37,9 +36,8 @@ There files are available by FTP from ftp://ftp.neb.com/pub/rebase/ which
 should allow automated fetching (the the update code and RanaConfig.py).
 In addition there are links on this HTML page which requires manual download
 and renaming of the files: http://rebase.neb.com/rebase/rebase.f37.html
-
 This Python file is intended to be used via the scripts in
-Scripts/Restriction/*.py only.
+`Scripts/Restriction/*.py` only.
 """
 
 from __future__ import print_function
@@ -52,7 +50,6 @@ import time
 import sys
 import shutil
 import optparse
-from functools import reduce
 
 from Bio.Seq import Seq
 from Bio.Alphabet import generic_dna
@@ -68,9 +65,6 @@ from Bio.Restriction.Restriction import Commercially_available, Not_available
 
 import Bio.Restriction.RanaConfig as config
 from rebase_update import RebaseUpdate
-from Bio.Restriction.Restriction import *
-
-__docformat__ = "restructuredtext en"
 
 
 enzymedict = {}
@@ -81,15 +75,18 @@ typedict = {}
 
 class OverhangError(ValueError):
     """Exception for dealing with overhang."""
+
     pass
 
 
 def regex(site):
-    """regex(site) -> string.
+    """Construct a regular expression (string) from a DNA sequence.
+
+    Example:
+
+        >>> regex('ABCGN')
+        'A[CGT]CG.'
 
-    Construct a regular expression from a DNA sequence.
-    i.e.:
-        site = 'ABCGN'   -> 'A[CGT]CG.'
     """
     reg_ex = str(site)
     for base in reg_ex:
@@ -105,20 +102,25 @@ def regex(site):
 
 
 def is_palindrom(sequence):
-    """is_palindrom(sequence) -> bool.
+    """Check whether the sequence is a palindrome or not (DEPRECATED).
 
-    True is the sequence is a palindrom.
-    sequence is a Seq object.
+    Deprecated alias for is_palindrome (with e at end).
     """
+    import warnings
+    from Bio import BiopythonDeprecationWarning
+    warnings.warn("is_palindrom is deprecated, please use "
+                  "is_palindrome instead.",
+                  BiopythonDeprecationWarning)
+    return is_palindrome(sequence)
+
+
+def is_palindrome(sequence):
+    """Check whether the sequence is a palindrome or not."""
     return str(sequence) == str(sequence.reverse_complement())
 
 
 def LocalTime():
-    """LocalTime() -> string.
-
-    LocalTime calculate the extension for emboss file for the current year and
-    month.
-    """
+    """Extension for emboss file for the current year and month."""
     t = time.gmtime()
     year = str(t.tm_year)[-1]
     month = str(t.tm_mon)
@@ -129,6 +131,7 @@ def LocalTime():
 
 class newenzyme(object):
     """construct the attributes of the enzyme corresponding to 'name'."""
+
     def __init__(cls, name):
         cls.opt_temp = 37
         cls.inact_temp = 65
@@ -272,17 +275,14 @@ class newenzyme(object):
 
 
 class TypeCompiler(object):
-    """Build the different types possible for Restriction Enzymes"""
+    """Build the different types possible for Restriction Enzymes."""
 
     def __init__(self):
         """TypeCompiler() -> new TypeCompiler instance."""
         pass
 
     def buildtype(self):
-        """TC.buildtype() -> generator.
-
-        build the new types that will be needed for constructing the
-        restriction enzymes."""
+        """Build new types that will be needed for constructing the enzymes."""
         baT = (AbstractCut, RestrictionType)
         cuT = (NoCut, OneCut, TwoCuts)
         meT = (Meth_Dep, Meth_Undep)
@@ -290,7 +290,6 @@ class TypeCompiler(object):
         ovT = (Unknown, Blunt, Ov5, Ov3)
         deT = (NotDefined, Defined, Ambiguous)
         coT = (Commercially_available, Not_available)
-        All = (baT, cuT, meT, paT, ovT, deT, coT)
         #
         #   Now build the types. Only the most obvious are left out.
         #   Modified even the most obvious are not so obvious.
@@ -332,6 +331,7 @@ class TypeCompiler(object):
             yield klass()
             n += 1
 
+
 start = """#!/usr/bin/env python
 #
 #      Restriction Analysis Libraries.
@@ -343,7 +343,7 @@ start = """#!/usr/bin/env python
 #\n\
 # This file is automatically generated - do not edit it by hand! Instead,
 # use the tool Scripts/Restriction/ranacompiler.py which in turn uses
-# Bio/Restriction/_Update/RestrictionCompiler.py
+# Scripts/Restriction/rebase_update.py and Bio/Restriction/RanaConfig.py
 #\n\
 # The following dictionaries used to be defined in one go, but that does
 # not work on Jython due to JVM limitations. Therefore we break this up
@@ -368,11 +368,7 @@ class DictionaryBuilder(object):
         self.proxy = ftp_proxy or config.ftp_proxy
 
     def build_dict(self):
-        """DB.build_dict() -> None.
-
-        Construct the dictionary and build the files containing the new
-        dictionaries.
-        """
+        """Construct dictionary and build files containing new dictionaries."""
         #
         #   first parse the emboss files.
         #
@@ -439,7 +435,6 @@ class DictionaryBuilder(object):
                 enzlst.append(name)
                 typedict[typename] = (bases, enzlst)
             for letter in cls.__dict__['suppl']:
-                supplier = suppliersdict[letter]
                 suppliersdict[letter][1].append(name)
         if not classdict or not suppliersdict or not typedict:
             print('One of the new dictionaries is empty.')
@@ -462,49 +457,52 @@ class DictionaryBuilder(object):
                   'classes...')
             results.write(start)
             results.write('rest_dict = {}\n')
+            results.write("\n\n")
             for name in sorted(classdict):
                 results.write("def _temp():\n")
                 results.write("    return {\n")
-                for key, value in classdict[name].items():
+                for key, value in sorted(classdict[name].items()):
                     results.write("        %s: %s,\n" %
                                   (repr(key), repr(value)))
                 results.write("    }\n")
+                results.write("\n\n")
                 results.write("rest_dict[%s] = _temp()\n" % repr(name))
-                results.write("\n")
+                results.write("\n\n")
             print('OK.\n')
             print('Writing the dictionary containing the suppliers data...')
             results.write('suppliers = {}\n')
+            results.write("\n\n")
             for name in sorted(suppliersdict):
                 results.write("def _temp():\n")
                 results.write("    return (\n")
                 for value in suppliersdict[name]:
                     results.write("        %s,\n" % repr(value))
                 results.write("    )\n")
+                results.write("\n\n")
                 results.write("suppliers[%s] = _temp()\n" % repr(name))
-                results.write("\n")
+                results.write("\n\n")
             print('OK.\n')
             print('Writing the dictionary containing the Restriction types...')
             results.write('typedict = {}\n')
+            results.write("\n\n")
             for name in sorted(typedict):
                 results.write("def _temp():\n")
                 results.write("    return (\n")
                 for value in typedict[name]:
                     results.write("        %s,\n" % repr(value))
                 results.write("    )\n")
+                results.write("\n\n")
                 results.write("typedict[%s] = _temp()\n" % repr(name))
-                results.write("\n")
+                results.write("\n\n")
             # I had wanted to do "del _temp" at each stage (just for clarity),
             # but that pushed the code size just over the Jython JVM limit. We
             # include one the final "del _temp" to clean up the namespace.
             results.write("del _temp\n")
-            results.write("\n")
             print('OK.\n')
         return
 
     def install_dict(self):
-        """DB.install_dict() -> None.
-
-        Install the newly created dictionary in the site-packages folder.
+        """Install the newly created dictionary in the site-packages folder.
 
         May need super user privilege on some architectures.
         """
@@ -554,10 +552,7 @@ class DictionaryBuilder(object):
         return
 
     def no_install(self):
-        """BD.no_install() -> None.
-
-        build the new dictionary but do not install the dictionary.
-        """
+        """Build the new dictionary but do not install the dictionary."""
         print('\n ' + '*' * 78 + '\n')
         # update = config.updatefolder
         try:
@@ -590,10 +585,7 @@ class DictionaryBuilder(object):
         return
 
     def lastrebasefile(self):
-        """BD.lastrebasefile() -> None.
-
-        Check the emboss files are up to date and download them if not.
-        """
+        """Check the emboss files are up to date and download them if not."""
         embossnames = ('emboss_e', 'emboss_r', 'emboss_s')
         #
         #   first check if we have the last update:
@@ -693,8 +685,9 @@ class DictionaryBuilder(object):
                 continue
 
     def parseline(self, line):
-        line = [line[0]] + [line[1].upper()] + [int(i) for i in line[2:9]] + \
-               line[9:]
+        line = [line[0]] + \
+            [line[1].upper()] + [int(i) for i in line[2:9]] + \
+            line[9:]
         name = line[0].replace("-", "_").replace(".", "_")
         site = line[1]  # sequence of the recognition site
         dna = Seq(site, generic_dna)
@@ -812,7 +805,7 @@ class DictionaryBuilder(object):
                 ovhgseq = site[fst3:] + (fst5 - size) * 'N'
             else:
                 #
-                #   should not happend
+                #   should not happen
                 #
                 raise ValueError('Error in #1')
         elif fst3 < 0 and size < fst5:
@@ -867,18 +860,19 @@ class DictionaryBuilder(object):
         rg = ''
         if is_palindrom(dna):
             line.append(True)
-            rg = ''.join(['(?P<', name, '>', regex(site.upper()), ')'])
+            rg = ''.join(['(?=(?P<', name, '>', regex(site.upper()), '))'])
         else:
             line.append(False)
-            sense = ''.join(['(?P<', name, '>', regex(site.upper()), ')'])
-            antisense = ''.join(['(?P<', name, '_as>',
-                                 regex(dna.reverse_complement()), ')'])
+            sense = ''.join(['(?=(?P<', name, '>', regex(site.upper()), '))'])
+            antisense = ''.join(['(?=(?P<', name, '_as>',
+                                 regex(dna.reverse_complement()), '))'])
             rg = sense + '|' + antisense
         #
         #   exact frequency of the site. (ie freq(N) == 1, ...)
         #
-        f = [4 / len(amb_dna[l]) for l in site.upper()]
-        freq = reduce(lambda x, y: x * y, f)
+        freq = 1
+        for base in site.upper():
+            freq *= 4.0 / len(amb_dna[base])
         line.append(freq)
         #
         #   append regex and ovhg1, they have not been appended before not to
@@ -929,6 +923,7 @@ class DictionaryBuilder(object):
         supplier = self.removestart(file3)
 
         i1, i2 = 0, 0
+        oldblock = None
         try:
             while True:
                 block, i1 = self.getblock(methfile, i1)
@@ -976,9 +971,10 @@ class DictionaryBuilder(object):
                     dna = Seq(enzymedict[other][0], generic_dna)
                     sense2 = regex(dna)
                     antisense2 = regex(dna.reverse_complement())
-                    sense = '(?P<' + other + '>' + sense1 + '|' + sense2 + ')'
-                    antisense = '(?P<' + other + '_as>' + antisense1 + '|' + \
-                                antisense2 + ')'
+                    sense = '(?=(?P<{}>{})|{})'.format(other, sense1, sense2)
+                    antisense = '(?=(?P<{}_as>{}|{}))'.format(other,
+                                                              antisense1,
+                                                              antisense2)
                     reg = sense + '|' + antisense
                     line[1] = line[1] + '|' + enzymedict[other][0]
                     line[-1] = reg
@@ -1017,6 +1013,7 @@ def standalone():
     options, args = parser.parse_args()
     return options, args
 
+
 if __name__ == '__main__':
     options, args = standalone()
     Builder = DictionaryBuilder(options.ftp_proxy)
