@@ -23,24 +23,29 @@ try:
     # Skip the test if reportlab is not installed
     import reportlab as r
     del r
-except:
+except Exception:
     raise MissingPythonDependencyError(
         "Install ReportLab if you want to use Bio.Graphics.")
 try:
     # Skip the test if reportlab is not installed
     from reportlab.graphics import renderPM
-except:
+    del renderPM
+except Exception:
     raise MissingPythonDependencyError(
         "Install ReportLab's renderPM module if you want to create "
         "bitmaps with Bio.Graphics.")
 try:
     # Skip the test if PIL is not installed
-    import Image as i
+    # The preferred PIL import has changed over time:
+    try:
+        from PIL import Image as i
+    except ImportError:
+        import Image as i
     del i
-except:
+except Exception:
     raise MissingPythonDependencyError(
-        "Install PIL (Python Imaging Library) if you want to create "
-        "bitmaps with Bio.Graphics.")
+        "Install Pillow or its predecessor PIL (Python Imaging Library) "
+        "if you want to create bitmaps with Bio.Graphics.")
 
 from reportlab.graphics.renderPM import RenderPMError
 
@@ -84,25 +89,26 @@ def real_test():
     # error here.
     except IndexError:
         pass
-    except IOError, err:
+    except IOError as err:
         if "encoder zip not available" in str(err):
             raise MissingExternalDependencyError(
                 "Check zip encoder installed for PIL and ReportLab renderPM")
         else:
             raise err
-    except RenderPMError, err :
-        if str(err).startswith("Can't setFont(") :
-            #TODO - can we raise the error BEFORE the unit test function
-            #is run? That way it can be skipped in run_tests.py
+    except RenderPMError as err:
+        if str(err).startswith("Can't setFont("):
+            # TODO - can we raise the error BEFORE the unit test function
+            # is run? That way it can be skipped in run_tests.py
             raise MissingExternalDependencyError(
                 "Check the fonts needed by ReportLab if you want "
                 "bitmaps from Bio.Graphics\n" + str(err))
-        else :
+        else:
             raise err
 
     return True
 
-#Run the actual test BEFORE the unittest stuff gets called
+
+# Run the actual test BEFORE the unittest stuff gets called
 real_test()
 
 
@@ -110,9 +116,10 @@ class ComparativeTest(unittest.TestCase):
     """Do tests for modules involved with comparing data."""
     def test_simple_scatter_plot(self):
         """Test creation of a simple PNG scatter plot."""
-        #Dummy method to show up via run_tests.py
+        # Dummy method to show up via run_tests.py
         pass
 
+
 if __name__ == "__main__":
-    runner = unittest.TextTestRunner(verbosity = 2)
+    runner = unittest.TextTestRunner(verbosity=2)
     unittest.main(testRunner=runner)
